/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.cdm.SwapSynonymAndTaxonAction;
import eu.etaxonomy.taxeditor.actions.name.ChangeSynonymToConceptRelationAction;
import eu.etaxonomy.taxeditor.actions.name.ChangeSynonymToMisapplicationAction;
import eu.etaxonomy.taxeditor.actions.name.CreateSynonymInGroupAction;
import eu.etaxonomy.taxeditor.actions.name.CreateNewTaxonFromSynonymAction;
import eu.etaxonomy.taxeditor.actions.name.MakeBasionymAction;
import eu.etaxonomy.taxeditor.actions.name.Messages;
import eu.etaxonomy.taxeditor.actions.name.RemoveSynonymAction;
import eu.etaxonomy.taxeditor.actions.name.UnmakeBasionymAction;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.operations.name.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.name.ChangeSynonymToMisappliedNameOperation;
import eu.etaxonomy.taxeditor.operations.name.UnmakeSynonymGroupBasionymOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.TaxonBasePropertySource;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class SynonymComposite extends NameComposite {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(SynonymComposite.class);
	
	private Synonym synonym;
	
	private IUndoContext undoContext;
	
	public SynonymComposite(Composite parent, IManagedForm form,
			Taxon taxon, Synonym synonym) {
		super(parent, form, synonym);
		
		this.taxon = taxon;
		this.synonym = synonym;
		
		this.undoContext = EditorController.getUndoContext(taxon);
		
		setIsDraggable(true);
		setFont(getViewerFont());
		setIndent(SYNONYM_INDENT);
		
		if (isHomotypic()) {
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			}	
		} else {
			if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
		}

		createMenu();
		
		initTextViewer(synonym);
	}
	
	private boolean isHomotypic() {
		HomotypicalGroup group = synonym.getHomotypicGroup();
		return group.equals(taxon.getHomotypicGroup()); 
	}
	
	public Synonym getSynonym() {
		return synonym;
	}

	public boolean setParent(Composite parent) {

		if (super.setParent(parent)) {	
			
			// Has this been moved to the misapplied names group?
			if (parent instanceof MisappliedGroupComposite) {
				
				IUndoableOperation operation = new ChangeSynonymToMisappliedNameOperation
						("change to misapplication", undoContext, taxon, synonym); //$NON-NLS-1$
				
				GlobalController.executeOperation(operation);
			}

			// Has this been moved to a HomotypicalGroup?
			if (parent instanceof HomotypicalGroupComposite) {
				
				HomotypicalGroup homotypicalGroup = 
						((HomotypicalGroupComposite)parent).getGroup();
				
				// Make sure we are not dropping synonym on its own group
				if (homotypicalGroup.equals(synonym.getHomotypicGroup())) {
					return true;
				}

				IUndoableOperation operation = new ChangeHomotypicGroupOperation
						("change type", undoContext, taxon, synonym, homotypicalGroup); //$NON-NLS-1$
				
				GlobalController.executeOperation(operation);
			}
			return true;

		}
		return false;
	}
	
	/**
	 * @param nameComposite
	 */
	private void createMenu() {

		ContextMenu contextMenu = createContextMenu();

		contextMenu.addAction(new RemoveSynonymAction(taxon, synonym));
		if (CdmUtil.isNameGroupBasionym(synonym.getName())) {
			contextMenu.addAction(new UnmakeBasionymAction(taxon, synonym));
		} else {
			contextMenu.addAction(new MakeBasionymAction(taxon, synonym));	
		}
		 // Add synonym to this synonyms homotypical group
		// FIXME not working with the new menu action 
//		contextMenu.addAction(new CreateSynonymInGroupAction(synonym.getHomotypicGroup()));
		contextMenu.addSeparator();
		contextMenu.addAction(new ChangeSynonymToMisapplicationAction(taxon, synonym));
		contextMenu.addAction(new ChangeSynonymToConceptRelationAction(taxon, synonym));
		contextMenu.addSeparator();				
		// Change this synonym to the taxon's accepted name
		Action swapAction = new SwapSynonymAndTaxonAction(taxon, synonym);
		contextMenu.addAction(swapAction);
		// TODO fix operation
		swapAction.setEnabled(false);
		
		// Make a new taxon with this synonym as the accepted name
		contextMenu.addAction(new CreateNewTaxonFromSynonymAction(taxon, synonym));
	}

	
	public IPropertySource getPropertySource() {
		return new TaxonBasePropertySource(synonym, "Synonym Name");
	}

	
	protected Font getViewerFont() {
		return SYNONYM_FONT;
	}

	@Override
	protected TaxonNameBase getName() {
		return synonym.getName();
	}
	
	public Object getData () {
		return getSynonym();
	}
}