/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.propertysheet.namerelations.wizard;

import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.labels.ILabelImageStrategy;
import eu.etaxonomy.taxeditor.labels.LabelImageProvider;
import eu.etaxonomy.taxeditor.model.NameHelper;

/**
 * @author p.ciardelli
 * @created 06.06.2008
 * @version 1.0
 */
public class ListNameRelationsWizardPage extends WizardPage {

	private Table nameRelationsTable;
	private WritableList nameRelationsList = new WritableList();
	private TaxonNameBase<?, ?> name;

	private Button btnRemove;
	private Button btnEdit;

	private NameRelationship selectedRelation;

	private LabelImageProvider labelProvider;

	public ListNameRelationsWizardPage(TaxonNameBase<?, ?> name, WritableList nameRelationsList) {
		super("");

		this.name = name;
		this.nameRelationsList = nameRelationsList;
		
		ILabelImageStrategy strategy = LabelImageProvider.getLabelStrategy(name);
		this.labelProvider = new LabelImageProvider(strategy);

		setTitle("Nomenclatural relations for \""
				+ NameHelper.getDisplayName(name) + "\".");
		setDescription("Select a relation and click \"Edit ...\" or \"Remove\", or click \"Add ...\" to create a new relation.");
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);

		setControl(container);

		btnEdit = new Button(container, SWT.NONE);
		btnEdit.setText("Edit ...");

		btnRemove = new Button(container, SWT.NONE);
		btnRemove.setText("Remove");
		btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				removeSelectedRelation();
			}
		});

		setEnableExistingRelationButtons(false);

		final Button btnAdd = new Button(container, SWT.NONE);
		final GridData gd_btnAdd = new GridData(SWT.RIGHT, SWT.CENTER, true,
				false);
		btnAdd.setLayoutData(gd_btnAdd);
		btnAdd.setText("Add ...");
		btnAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addRelation();
			}
		});

		final TableViewer tableViewer = new TableViewer(container, SWT.BORDER | SWT.SINGLE);

		nameRelationsTable = tableViewer.getTable();
		nameRelationsTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 3, 1));
		nameRelationsTable.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// TODO: As long as onNameBase.removeNameRelationship() doesn't work, "Remove" stays disabled ... 
				setEnableExistingRelationButtons(true);
				if (e.item.getData() instanceof NameRelationship) {
					setSelectedRelation((NameRelationship) e.item.getData());
				}
			}
		});

		ObservableListContentProvider providerList = new ObservableListContentProvider();
		tableViewer.setContentProvider(providerList);

		IObservableMap[] providerMaps = BeansObservables.observeMaps(
				providerList.getKnownElements(), NameRelationship.class,
				new String[] { "fromName", "toName", "type" });
		tableViewer.setLabelProvider(new ObservableMapLabelProvider(
				providerMaps) {
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof NameRelationship) {
					NameRelationship nameRelationship = (NameRelationship) element;
					return getRelationshipString(nameRelationship);
				}
				return "";
			}

			public Image getColumnImage(Object element, int columnIndex) {
				if (element instanceof NameRelationship) {
					return labelProvider.getNameRelationTypeImage(((NameRelationship) element).getType());
				}
				return null;
			}
		});
		tableViewer.setInput(nameRelationsList);
		tableViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (((StructuredSelection) event.getSelection())
						.getFirstElement() instanceof NameRelationship) {
					// FIXME nothing is happening here
					NameRelationship relation = (NameRelationship) ((StructuredSelection) event
							.getSelection()).getFirstElement();
					// Editing existing relations disabled
//					createRelationWizard(relation);
				}
			}
		});
	}

	/**
	 * 
	 */
	protected void removeSelectedRelation() {
		if (getSelectedRelation() != null) {
			nameRelationsList.remove(getSelectedRelation());
			setSelectedRelation(null);
		}
	}

	/**
	 * 
	 */
	protected void addRelation() {
		Wizard wizard = new NameRelationWizard(name, nameRelationsList);
		
		WizardDialog dialog = new WizardDialog(EditorUtil.getShell(), wizard);
		dialog.create();
		dialog.open();
	}

	private void setSelectedRelation(NameRelationship selectedRelation) {
		this.selectedRelation = selectedRelation;
	}

	private NameRelationship getSelectedRelation() {
		return selectedRelation;
	}

	protected void setEnableExistingRelationButtons(boolean enabled) {
		btnRemove.setEnabled(enabled);
		btnEdit.setEnabled(false);
	}

	private String getRelationshipString(NameRelationship nameRelationship) {
		String fromName = null;
		String type = null;
		String toName = null;
		if (nameRelationship.getFromName() == null) {
			fromName = "";
		} else {
			fromName = NameHelper.getDisplayName(nameRelationship.getFromName());
		}
		if (nameRelationship.getType() == null) {
			type = "";
		} else {
			type = labelProvider.getNameRelationTypeLabel(nameRelationship.getType());			
		}
		if (nameRelationship.getToName() == null) {
			toName = "";
		} else {
			toName = NameHelper.getDisplayName(nameRelationship.getToName());
		}
		return "\"" + fromName + "\" is \"" + type + "\" of \"" + toName + "\"";
	}
}
